/*	
 *	relax - an editor for creating HTML documents
 *	
 *	Copyright (C) 1994 by the Department of Computer Science at the TUM
 *	(Munich University of Technology, Germany)
 *	
 *	Authors: Bernhard Blaette, Michael Tiefenbacher
 *	
 *	This program is partly based on the NCSA Mosaic software.
 *	NCSA Mosaic software, both binary and source (hereafter, Software) is
 *	copyrighted by The Board of Trustees of the University of Illinois
 *	(UI), and ownership remains with the UI.
 *	The editor "relax" uses the 'htmlWidgetClass' and the 'HTMLSetText' 
 *	procedure of the 'HTML.h' include file for parsing the edited text 
 *	into a Mosaic formatted style. Moreover, 'HTMLamp.h' is included for
 *	the special character popup. 
 *	Certainly, "relax" is not the original NCSA Mosaic but uses some of
 *	its Software for such derivative work.
 *	
 *	Permission to use, copy, modify, and distribute this software and its 
 *	documentation for any purpose and without fee is hereby granted,
 *	provided that this comment and the authors' names are retained and
 *	appear in all copies and that both that copyright notice and this 
 *	permission notice appear in supporting documentation.
 *	
 *	This software is provided 'as is' without express or implied warranty.
 *	In no event shall the authors or the TUM be liable for errors contained 
 *	in the software, or for direct, indirect, special, incidental or 
 *	consequential damages with the use of the software.
 *	
 */
#include <stdio.h>
#include <sys/stat.h>

#include <relax.icon>

#include <relax_widget-stuff.h>
#include <relax_global-defines.h>
#include <relax_menu-specs.h>

/*----------------------------------------------------------------------------*/
Widget W_ScrolledText;
Widget W_FileLabel;
Widget W_ButtonPaste;
Widget W_MenuPaste;
Widget W_MainWindow;
Widget W_MenuBar;
Widget W_ButtonForm;       

Bool  Info_Flag=FALSE;

char*	HomeDir;
char*	FileNameImageHist;
char*	FileNameLinkHist;

void 	CreateMenu();
void 	CreateBottomButtons();


/*----------------------------------------------------------------------*/
/*INFO Callback   */
/*----------------------------------------------------------------------*/

void
Close_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	Widget close;
	
	Info_Flag=FALSE;

	close = (Widget) client_data;

	XtDestroyWidget(close);
}

void 
CreateInfo(message)
char*   message;
{
	Widget 		infobox, button, close,
			dialogShell, frame, form;
	Arg  		args[10];
     	register int  	n;
	XmString	button_string = NULL;
	XmString	message_string = NULL;

	n = 0;
	dialogShell = XmCreateDialogShell(
			W_MenuBar, "INFO-Box", args, n);
	n = 0;
	frame = XmCreateFrame(dialogShell, "INFO-Box", args, n);

	n = 0;
	form = XmCreateFrame(frame, "form", args, n);


	message_string = XmStringCreateLtoR(message,  XmFONTLIST_DEFAULT_TAG);	
	button_string = XmStringCreateLtoR("Close",  XmFONTLIST_DEFAULT_TAG);

	n = 0;
	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++; 
  	XtSetArg(args[n], XmNokLabelString, button_string); n++;
	XtSetArg(args[n], XmNmessageString, message_string); n++;	

	infobox = XmCreateMessageBox( form, "INFO-Box", args, n );

	button = XmMessageBoxGetChild (infobox, XmDIALOG_CANCEL_BUTTON);
	XtUnmanageChild (button);
	button = XmMessageBoxGetChild (infobox, XmDIALOG_HELP_BUTTON);
	XtUnmanageChild (button);

	close = XmMessageBoxGetChild(infobox, XmDIALOG_OK_BUTTON);
	XtAddCallback (close, XmNactivateCallback, Close_CB, dialogShell);

	XtManageChild (infobox);
	XtManageChild(frame);
	XtManageChild(form);

	XmStringFree(message_string);
	XmStringFree(button_string);
}

void 
GetInfo_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	char		infomessage[1000];	
	
	/*Generate INFO-Message to display  */
	sprintf(infomessage, 
	"This is the HTML - Editor \n\
	RELAX  Version 1.0 \n\
	\n\
	created by \n\
	Bernhard Blaette  and  Michael Tiefenbacher \n\
	\n\
	EMAIL: \n\
	blaetteb@informatik.tu-muenchen.de \n\
	tiefenba@informatik.tu-muenchen.de \0");	
	

	if (Info_Flag == True)
	{
		return;
	}

	Info_Flag = TRUE;

	CreateInfo(infomessage);
}

void 
GetHelp_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	char* 	filename = malloc(100);

	strcpy( filename, HomeDir );
	strcat( filename, "/.relax_helptext" );

	system("xterm -sb -sl 500 -title 'relax HELP' -e man relax &");
}


/******************************************************************************/
/*  MAIN								      */
/******************************************************************************/

void
main(argc, argv)
int argc;
char* argv[];
{
    XtAppContext app_context;

    Widget        form, topLevel, getInfo;
    Widget        label1;
    Widget	  separator, menu_pane, about, help;
    Arg           args[20];
    int           n;
    XmString      label_string, textf_label, infobutton;

/*----------------------------------------------------------------------------*/
/*  Keybindings								      */
/*----------------------------------------------------------------------------*/
     static XtActionsRec acts[] = {
     {"EnableUndo", 	(XtActionProc) EnableUndo}
     }; 

     static XtTranslations  parsed_translations;

       static char translations[] = "\
     #override<Key>Return: 	process-return()\n\
     <Key>osfBackSpace: 	EnableUndo() delete-previous-character()\n\
     <Key>osfRight: 		forward-character()\n\
     <Key>osfLeft: 		backward-character()\n\
     <Key>osfUp: 		process-up()\n\
     <Key>osfDown: 		process-down()\n\
     <Key>Tab: 			process-tab()\n\
     <Key>osfPageUp: 		previous-page()\n\
     <Key>osfPageDown: 		next-page()\n\
     <Key>osfBeginLine: 	beginning-of-line()\n\
     <Key>osfEndLine: 		end-of-line()\n\
     <Key>Delete: 		EnableUndo() delete-next-character()\n\
     <Key>:			self-insert()";
/*----------------------------------------------------------------------------*/

    XtSetLanguageProc (NULL, (XtLanguageProc)NULL, NULL);

/*USEFUL */
    if( (HomeDir = getenv("HOME")) == NULL )
    {	HomeDir = ".";
    }

/*TOPLEVEL  */
    n = 0;    

    topLevel = XtAppInitialize(
            &app_context, "Relax", NULL, 0, &argc, argv, NULL, args, n);   

    n = 0;    
    XtSetArg (args[n], XtNiconPixmap, XCreateBitmapFromData(
			XtDisplay(topLevel), XtScreen(topLevel)->root,
			relax_bits, relax_width, relax_height)); 	n++;
    XtSetValues (topLevel, args, n);

    XtAppAddActions(app_context, acts, XtNumber (acts));

    parsed_translations = XtParseTranslationTable(translations);

/*----------------------------------------------------------------------------*/

/*FORM  */
    n = 0;

    form = XmCreateForm(topLevel, "form", args, n);  		
    XtManageChild (form);

/*LABEL FUER FENSTER  */
    label_string = XmStringCreate(
            "HTML - Editor relax", XmFONTLIST_DEFAULT_TAG);
    n = 0;
    XtSetArg(args[n], XmNlabelString,label_string); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++; 
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopOffset, 3); n++;
    XtSetArg(args[n], XmNbottomOffset, 3); n++;

    label1 = XmCreateLabel(form, "HTMLlabel", args, n);

    XtManageChild(label1);
    XmStringFree(label_string);

/*SEPARATOR  */
    n = 0;
    XtSetArg(args[n], XmNseparatorType, XmSHADOW_ETCHED_IN); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopWidget, label1); n++;

    separator = XmCreateSeparator(form, "separator", args, n);

    XtManageChild(separator);   

/*MESSAGELABEL - FILENAME  */
    textf_label = XmStringCreate("*unknown*", XmFONTLIST_DEFAULT_TAG);    
    n = 0;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg(args[n], XmNtopWidget, separator); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;                      XtSetArg(args[n], XmNalignment, XmALIGNMENT_CENTER); n++;                      XtSetArg(args[n], XmNlabelString, textf_label); n++;
 
    W_FileLabel = XmCreateLabel(form, "FileLabel", args, n);

    XtManageChild(W_FileLabel);
    XmStringFree(textf_label);
/*----------------------------------------------------------------------*/

/*BUTTONFORM */
    n=0;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;

    W_ButtonForm = XtCreateManagedWidget(
            "form", xmFormWidgetClass, form, args, n);  

/*MAINWINDOW  */
    n = 0;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++; 
    XtSetArg(args[n], XmNbottomWidget, W_ButtonForm); n++; 
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg(args[n], XmNtopWidget, W_FileLabel); n++;

    W_MainWindow = XtCreateManagedWidget(
            "MainWindow", xmMainWindowWidgetClass, form, args, n);

/*MENUBAR  */
    W_MenuBar = XmCreateMenuBar(
            W_MainWindow, "MenuBar", NULL, 0);

    XtManageChild(W_MenuBar);

/*INFOBUTTON WITH SPECIAL POSITION  */

    n = 0;
    menu_pane = XmCreatePulldownMenu (W_MenuBar, "menu_pane", args, n);

    n = 0;
    XtSetArg(args[n], XmNmnemonic, 'H'); n++;
    help = XmCreatePushButton (menu_pane, "Help", args, n);
    XtManageChild (help);
    XtAddCallback (help, XmNactivateCallback, GetHelp_CB, NULL);

    n = 0;
    XtSetArg(args[n], XmNmnemonic, 'A'); n++;
    about = XmCreatePushButton (menu_pane, "About", args, n);
    XtManageChild (about);
    XtAddCallback (about, XmNactivateCallback, GetInfo_CB, NULL);

    infobutton = XmStringCreateSimple ("Info");
    n = 0;
    XtSetArg (args[n], XmNlabelString, infobutton);  n++;
    XtSetArg(args[n], XmNmnemonic, 'I'); n++;
    XtSetArg (args[n], XmNsubMenuId, menu_pane);  n++;

    getInfo = XmCreateCascadeButton (W_MenuBar, NULL, args, n);
    XtManageChild (getInfo);
    XmStringFree(infobutton);

    n = 0;
    XtSetArg (args[n], XmNmenuHelpWidget, getInfo); n++;
    XtSetValues(W_MenuBar, args, n);

/*SET MAINWINDOWAREAS */
    XmMainWindowSetAreas (
		W_MainWindow, W_MenuBar, NULL, NULL, NULL, NULL);

/*----------------------------------------------------------------------*/

/*SCROLLED TEXT  */
    n = 0;

    XtSetArg(args[n], XmNscrollVertical, True); n++;
    XtSetArg(args[n], XmNscrollHorizontal, True); n++;
    XtSetArg(args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
    XtSetArg(args[n], XmNvalue, ""); n++;
    XtSetArg(args[n], XmNtranslations, parsed_translations); n++;

    W_ScrolledText = XmCreateScrolledText(
		W_MainWindow, "ScrolledText", args, n);

    XtManageChild(W_ScrolledText);

    CheckBuffer = &NullChar;
/*----------------------------------------------------------------------*/

/*LOAD HISTORIES  */


    FileNameImageHist = malloc( strlen(HomeDir) + strlen("/.relax_image_hist")
				 + 1 );

    FileNameLinkHist = malloc( strlen(HomeDir) + strlen("/.relax_link_hist")
				 + 1 );

    strcpy( FileNameImageHist, HomeDir ); 
    strcat( FileNameImageHist, "/.relax_image_hist");

    strcpy( FileNameLinkHist, HomeDir ); 
    strcat( FileNameLinkHist, "/.relax_link_hist");

    LoadLists(Image_List, FileNameImageHist, &Image_Counter);
    LoadLists(Link_List, FileNameLinkHist, &Link_Counter); 
/*----------------------------------------------------------------------*/

/*MENUCREATION - LOOP  */
    CreateMenu (W_MenuBar);
 		
    CreateBottomButtons (W_ButtonForm);

/*----------------------------------------------------------------------*/

    XtRealizeWidget(topLevel);

    XtAppMainLoop(app_context);  


}

/******************************************************************************/
/*  END MAIN								      */
/******************************************************************************/

/*LOOP FOR MENU  */

static void 
CreateMenu(parent)
Widget parent;
{
    Widget entry, command, pull;
    Arg args[10];
    int n,i,j;
    char mnemonic;
    char acceltext[100];
    char accelkey[100];

    i = 0;
    while(!(Button_List[i].ItemsPtr == NULL)) 
    {
	n = 0;
	pull = XmCreatePulldownMenu(parent, "pull", args, n);


	command = XtCreateManagedWidget( 
		Button_List[i].ButtonName, 
		xmCascadeButtonWidgetClass, parent,
		NULL, NULL);

	XtVaSetValues(command, XmNsubMenuId, pull, NULL);

	mnemonic = Button_List[i].ButtonMnemonic;
	if ( mnemonic != NO_MNEMO )
	{	XtVaSetValues(command, XmNmnemonic, mnemonic, NULL);
	}


    	j = 0;
 	while(!((*Button_List[i].ItemsPtr)[j].ItemProc == NULL))
    	{
	    char* itemname;

	    itemname = (*Button_List[i].ItemsPtr)[j].ItemName;
	    entry = XtVaCreateManagedWidget( itemname,
			xmPushButtonWidgetClass, pull,
			NULL);

	    XtAddCallback(entry, XmNactivateCallback,
			(*Button_List[i].ItemsPtr)[j].ItemProc,	
			(XtPointer) itemname);

	    mnemonic = (*Button_List[i].ItemsPtr)[j].ItemMnemonic;
	    if ( mnemonic != NO_MNEMO )
	    {	XtVaSetValues(entry, XmNmnemonic, mnemonic, NULL);
	    }

	    strcpy(acceltext, (*Button_List[i].ItemsPtr)[j].ItemAccelText);
	    if (strcmp(acceltext, NO_ACCEL) != 0)
	    {   XmString accelStr;
		accelStr = XmStringCreateSimple (acceltext);
		XtVaSetValues(entry, XmNacceleratorText, accelStr, NULL);
		XmStringFree (accelStr);
	    }

	    strcpy(accelkey, (*Button_List[i].ItemsPtr)[j].ItemAccelKey);
	    if (strcmp(accelkey, NO_ACCEL) != 0)
	    {	XtVaSetValues(entry, XmNaccelerator, accelkey, NULL);
	    }


	    if 	(strcmp(itemname, "Paste")==0)
	    {
		XtSetSensitive(entry, False);
		W_MenuPaste = entry;
	    }
	    j++;
	}	
	i++;
    }
}

/*LOOP FOR BUTTONS AT THE BOTTOM  */

void
CreateBottomButtons(bottombar)
Widget bottombar;
{ 
    Widget 	button;
    Arg 	args[20];
    int 	n, k;

    k = 0;
    while(!(BottomButton_List[k].ItemProc == NULL))
    {
	n = 0;
        XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
     	XtSetArg(args[n], XmNbottomOffset, 2); n++;        										
	XtSetArg(args[n], XmNtopOffset, 2); n++;

	if (k == 0)
	{
		XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	}
	else
	{	
		XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
		XtSetArg(args[n], XmNleftWidget, button); n++;
	}

	button = XmCreatePushButton(bottombar, BottomButton_List[k].ItemName, args, n);
    	XtManageChild(button);
	
	XtAddCallback(button, XmNactivateCallback,
	 		BottomButton_List[k].ItemProc, 
			(XtPointer)BottomButton_List[k].ItemName);			
		if (strcmp(BottomButton_List[k].ItemName,"Paste")==0)
		{
		 	XtSetSensitive(button, False);
			W_ButtonPaste = button;
		}
	k++;
    }    
}

/*END OF LOOPS  */










