/*	
 *	relax - an editor for creating HTML documents
 *	
 *	Copyright (C) 1994 by the Department of Computer Science at the TUM
 *	(Munich University of Technology, Germany)
 *	
 *	Authors: Bernhard Blaette, Michael Tiefenbacher
 *	
 *	This program is partly based on the NCSA Mosaic software.
 *	NCSA Mosaic software, both binary and source (hereafter, Software) is
 *	copyrighted by The Board of Trustees of the University of Illinois
 *	(UI), and ownership remains with the UI.
 *	The editor "relax" uses the 'htmlWidgetClass' and the 'HTMLSetText' 
 *	procedure of the 'HTML.h' include file for parsing the edited text 
 *	into a Mosaic formatted style. Moreover, 'HTMLamp.h' is included for
 *	the special character popup. 
 *	Certainly, "relax" is not the original NCSA Mosaic but uses some of
 *	its Software for such derivative work.
 *	
 *	Permission to use, copy, modify, and distribute this software and its 
 *	documentation for any purpose and without fee is hereby granted,
 *	provided that this comment and the authors' names are retained and
 *	appear in all copies and that both that copyright notice and this 
 *	permission notice appear in supporting documentation.
 *	
 *	This software is provided 'as is' without express or implied warranty.
 *	In no event shall the authors or the TUM be liable for errors contained 
 *	in the software, or for direct, indirect, special, incidental or 
 *	consequential damages with the use of the software.
 *	
 */
#define ENDE ""
#define NO_MNEMO '\0'
#define NO_ACCEL ""

/*	Menu-Structure:
 *		Menu Buttons: 		'Buttons'
 *		Submenu Buttons: 	'Items'
 *              Buttons at the Bottom: 	'ButtonList'
 */

typedef struct
{	char* 		ItemName;
	void		(*ItemProc)();
	char		ItemMnemonic;
	char*		ItemAccelText;
	char*		ItemAccelKey;
} Items[];

typedef struct
{	char*		ButtonName;
	Items* 		ItemsPtr;
	char		ButtonMnemonic;
} Buttons[];

typedef struct
{	char* 		ItemName;
	void		(*ItemProc)();
} ButtonList[];


/*List of Functions:  */
 
extern 	void	Menu_QuitCheck();
extern 	void	Menu_SaveDialog();
extern 	void	Menu_LoadCheck();
extern 	void	Menu_Parse();
extern 	void	Menu_SaveText();
extern 	void	Menu_CharSelDialog();
extern 	void	Menu_Cut();
extern 	void	Menu_Copy();
extern 	void	Menu_Paste();
extern 	void	Menu_Delete();
extern 	void	Menu_Tags();
extern 	void	Menu_ButtonCharSel();
extern 	void	Menu_LinkToName();
extern 	void	Menu_LinkToFile();
extern 	void	Menu_LinkToImage();
extern 	void	Menu_SetName();
extern 	void	Menu_NewCheck();
extern 	void	Menu_Undo();


/*Creation of the Menue:  */

Items File_items = 
{    	{	"New", 		Menu_NewCheck,	'N',	
				NO_ACCEL,	NO_ACCEL
	},
	{	"Load", 	Menu_LoadCheck,	'L',
				NO_ACCEL,	NO_ACCEL				
	},
	{	"Save", 	Menu_SaveText,	NO_MNEMO,
				NO_ACCEL,	NO_ACCEL
	},
	{	"Save As ..", 	Menu_SaveDialog,'S',
				NO_ACCEL,	NO_ACCEL
	},
	{	"Quit",		Menu_QuitCheck,	'Q',
				"Ctrl+Q",	"Ctrl<Key>Q:"
	},
	{	ENDE,		NULL,		NO_MNEMO,
				NO_ACCEL,	NO_ACCEL
	}
};

Items Edit_items = 
{	{	"Parse",	Menu_Parse,	'P',
				"Ctrl+Z",	"Ctrl<Key>Z:"
	},    	
	{	"Undo",		Menu_Undo,	'U',
				"Ctrl+U",	"Ctrl<Key>U:"
	},
	{	"Cut", 		Menu_Cut,	't',
				"Ctrl+X",	"Ctrl<Key>X:"
	},
	{	"Copy", 	Menu_Copy,	'C',
				"Ctrl+C",	"Ctrl<Key>C:"
	},
	{	"Paste", 	Menu_Paste,	'a',
				"Ctrl+V",	"Ctrl<Key>V:"
	},
	{	"Delete", 	Menu_Delete,	'D',
				"Delete",	NO_ACCEL
	},
	{	ENDE,		NULL,		NO_MNEMO,
				NO_ACCEL,	NO_ACCEL
	}
};

Items Text_items = 
{	{	"Title",	Menu_Tags,	'T',
				NO_ACCEL,	NO_ACCEL
	},    	
	{	"Bold",		Menu_Tags,	'B',
				"Ctrl+B",	"Ctrl<Key>B:"
	},    	
	{	"Emphasized",	Menu_Tags,	'E',
				"Ctrl+E",	"Ctrl<Key>E:"
	},
	{	"Strong",	Menu_Tags,	'S',
				NO_ACCEL,	NO_ACCEL
	},    	
	{	"Italics", 	Menu_Tags,	'I',
				"Ctrl+I",	"Ctrl<Key>I:"
	},
	{	"Citation", 	Menu_Tags,	'C',
				NO_ACCEL,	NO_ACCEL
	},
	{	"Typewriter", 	Menu_Tags,	'y',
				"Ctrl+Y",	"Ctrl<Key>Y:"
	},
	{	"Code", 	Menu_Tags,	'o',
				NO_ACCEL,	NO_ACCEL
	},
	{	"Sample", 	Menu_Tags,	'a',
				NO_ACCEL,	NO_ACCEL
	},
	{	"Keyboard", 	Menu_Tags,	'K',
				"Ctrl+K",	"Ctrl<Key>K:"
	},
	{	"Variable", 	Menu_Tags,	'V',
				NO_ACCEL,	NO_ACCEL
	},
	{	"Comment", 	Menu_Tags,	'm',
				"Ctrl+M",	"Ctrl<Key>M:"
	},
	{	"Special Chars",Menu_CharSelDialog,	'p',
				"Ctrl+S",	"Ctrl<Key>S:"
	},
	{	ENDE,		NULL,		NO_MNEMO,
				NO_ACCEL,	NO_ACCEL
	}
};

Items Heading_items = 
{    	{	"Type H1", 	Menu_Tags,	'1',
				"Ctrl+1",	"Ctrl<Key>1:"
	},
	{	"Type H2", 	Menu_Tags,	'2',
				"Ctrl+2",	"Ctrl<Key>2:"
	},
	{	"Type H3", 	Menu_Tags,	'3',
				"Ctrl+3",	"Ctrl<Key>3:"
	},
	{	"Type H4", 	Menu_Tags,	'4',
				"Ctrl+4",	"Ctrl<Key>4:"
	},
	{	"Type H5",	Menu_Tags,	'5',
				"Ctrl+5",	"Ctrl<Key>5:"
	},
	{ 	"Type H6", 	Menu_Tags,	'6',
				"Ctrl+6",	"Ctrl<Key>6:"
	},
	{	ENDE,		NULL,		NO_MNEMO,
				NO_ACCEL,	NO_ACCEL
	}
};

Items List_items = 
{	{	"Unnumbered",	Menu_Tags,	'U',
				NO_ACCEL,	NO_ACCEL
	},
	{ 	"Numbered",	Menu_Tags,	'N',
				NO_ACCEL,	NO_ACCEL
	},
	{	"Descriptive",	Menu_Tags,	'S',
				NO_ACCEL,	NO_ACCEL
	},
	{	"Add <LI>",	Menu_Tags,	'L',
				"Ctrl+L",	"Ctrl<Key>L:"
	},
	{	"Add <DT>, <DD>",Menu_Tags,	'T',
				"Ctrl+D",	"Ctrl<Key>D:"
	},
	{	ENDE,		NULL,		NO_MNEMO,
				NO_ACCEL,	NO_ACCEL
	}
};

Items Block_items = 
{	{	"Paragraph",	Menu_Tags,	'P',
				"Ctrl+P",	"Ctrl<Key>P:"
	},
	{	"New Line",	Menu_Tags,	'N',
				"Ctrl+N",	"Ctrl<Key>N:"
	},
	{	"Blockquote",	Menu_Tags,	'B',
				"Ctrl+O",	"Ctrl<Key>O:"
	},
	{	"Preformatted",	Menu_Tags,	'r',
				"Ctrl+F",	"Ctrl<Key>F:"
	},
	{	"Listing",	Menu_Tags,	'L',
				"Ctrl+G",	"Ctrl<Key>G:"
	},
	{	"Address",	Menu_Tags,	'A',
				"Ctrl+A",	"Ctrl<Key>A:"
	},
	{	"Horizontal Ruler",Menu_Tags,	'H',
				"Ctrl+H",	"Ctrl<Key>H:"
	},
	{	ENDE,		NULL,		NO_MNEMO,
				NO_ACCEL,	NO_ACCEL
	}
};

Items Links_items = 
{	{	"LinkToName", 		Menu_LinkToName,'A',
					NO_ACCEL,	NO_ACCEL
	},
	{	"Set Name",     	Menu_SetName,	'S',
					NO_ACCEL,	NO_ACCEL
	},	
	{	"LinkToFile", 		Menu_LinkToFile,'L',
					NO_ACCEL,	NO_ACCEL
	},
	{	"LinkToImage", 		Menu_LinkToImage,'I',
					NO_ACCEL,	NO_ACCEL
	},
	{	ENDE,			NULL,		NO_MNEMO,
					NO_ACCEL,	NO_ACCEL
	}	
};


Buttons Button_List =
{	{	"File",		File_items,		'F'
	},	
	{	"Edit",		Edit_items,		'E'
	},
	{	"Text",		Text_items,		'T'
	},	
	{	"Headlines",	Heading_items,		'H'
	},
	{	"Lists",	List_items,		'L'
	},
	{	"Block",	Block_items,		'B'
	},
	{	"Links",	Links_items,		'k'
	},	
	{	ENDE,		NULL,			NO_MNEMO	
	}
};

/*List for Bottom Buttons */

ButtonList BottomButton_List=
{	{	"Parse",		Menu_Parse
	},
	{	"Cut",			Menu_Cut
	},
	{	"Copy",			Menu_Copy
	},
	{	"Paste",		Menu_Paste
	},
	{	"Undo",			Menu_Undo
	},
	{	"New Line",		Menu_Tags
	},
	{	"Paragraph",		Menu_Tags	
	},
	{	"",			Menu_ButtonCharSel
	},
	{	"",			Menu_ButtonCharSel
	},
	{	"",			Menu_ButtonCharSel
	},
	{	"",			Menu_ButtonCharSel
	},
	{	"",			Menu_ButtonCharSel
	},
	{	"",			Menu_ButtonCharSel
	},
	{	"",			Menu_ButtonCharSel
	},
	{	"&",			Menu_ButtonCharSel
	},
	{	"<",			Menu_ButtonCharSel
	},
	{	">",			Menu_ButtonCharSel
	},
	{	ENDE,			NULL
	}
};	
