/*	
 *	relax - an editor for creating HTML documents
 *	
 *	Copyright (C) 1994 by the Department of Computer Science at the TUM
 *	(Munich University of Technology, Germany)
 *	
 *	Authors: Bernhard Blaette, Michael Tiefenbacher
 *	
 *	This program is partly based on the NCSA Mosaic software.
 *	NCSA Mosaic software, both binary and source (hereafter, Software) is
 *	copyrighted by The Board of Trustees of the University of Illinois
 *	(UI), and ownership remains with the UI.
 *	The editor "relax" uses the 'htmlWidgetClass' and the 'HTMLSetText' 
 *	procedure of the 'HTML.h' include file for parsing the edited text 
 *	into a Mosaic formatted style. Moreover, 'HTMLamp.h' is included for
 *	the special character popup. 
 *	Certainly, "relax" is not the original NCSA Mosaic but uses some of
 *	its Software for such derivative work.
 *	
 *	Permission to use, copy, modify, and distribute this software and its 
 *	documentation for any purpose and without fee is hereby granted,
 *	provided that this comment and the authors' names are retained and
 *	appear in all copies and that both that copyright notice and this 
 *	permission notice appear in supporting documentation.
 *	
 *	This software is provided 'as is' without express or implied warranty.
 *	In no event shall the authors or the TUM be liable for errors contained 
 *	in the software, or for direct, indirect, special, incidental or 
 *	consequential damages with the use of the software.
 *	
 */
#include <stdio.h>

#include <relax_widget-stuff.h>
#include <relax_global-defines.h>

#include <HTML.h>

/*----------------------------------------------------------------------------*/
XmTextPosition 	UndoCursorPos;
char*  		UndoBuffer = &NullChar;
XmTextPosition 	RedoCursorPos;
char*  		RedoBuffer = &NullChar;
Bool		ParsedText_Flag=FALSE;

Widget 		W_HtmlWidget;
Widget		W_DocumentTitle;

void 		EnableUndo();


/*----------------------------------------------------------------------*/
/*Scrolled Text Sensitive  */
/*----------------------------------------------------------------------*/

void
ScrolledTextSetSensitive()
{        
	XtSetSensitive(W_ButtonForm, False);
	XtSetSensitive(W_ScrolledText, True);
	XmTextSetEditable(W_ScrolledText, True);   
	XtSetSensitive(W_ButtonForm, True);
}

/*----------------------------------------------------------------------*/
/*GetTagged  */
/*----------------------------------------------------------------------*/

char*
GetTagged(tag, pos)
char* tag;
XmTextPosition pos;
{
	XmTextPosition act, textlen;
	char* text;
	char* lefterg=NULL;
	char* righterg=NULL;
	char* erg=NULL;
	int taglen, erglen;
	
	text=XmTextGetString(W_ScrolledText);
	textlen=XmTextGetLastPosition(W_ScrolledText);
	act=pos;
	taglen=strlen(tag);

	while (act<(textlen-taglen-1)
		&& 
		!(text[act] == '<' 
			&& text[act+taglen+1]=='>' 
			&& (strncasecmp(&(text[act+1]),tag,taglen)==0)
		)
	      )	act++;
	if (act<(textlen-taglen-2))
	{
		lefterg=&(text[act+taglen+2]);
		while (act<(textlen-taglen-2)
			&& 
			!(text[act] == '<' 
				&& text[act+1] == '/' 
				&& text[act+taglen+2]=='>' 
				&& (strncasecmp(&(text[act+2]),tag,taglen)==0)
			)
	      	) act++;
		if (act<(textlen-taglen-2))
		{	
			righterg=&(text[act-1]);
			erglen = righterg-lefterg+1;
			if (erglen>=0)
			{	erg = malloc(erglen+1);
				strncpy(erg, lefterg, erglen);
				erg[erglen]='\0';
			}
		}
	}	
	XtFree(text);
	return (erg);
}
	

char*
GetDocumentTitle()
{
	char* title;
	title=GetTagged("title", 0);
	return(title);
}
	

/*----------------------------------------------------------------------*/
/*SetText  */
/*----------------------------------------------------------------------*/

void 
SetTextCancel_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{	
	Widget Cancel, SetTextForm;

	Cancel = (Widget) widget;
	SetTextForm = XtParent(XtParent(Cancel));

	XtDestroyWidget(XtParent(XtParent(SetTextForm)));

	ParsedText_Flag=FALSE;
}


void
Menu_Parse(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{   
 	Arg           args[20];
    	int           n;
	Widget	      SetTextForm, SetTextLabel, dialogShell, frame;
	Widget	      Separator, Separator2, Cancel, Cancelbuttonform;	
	char* 	      inhalt;
	char*	      documenttitle;
 	XmString      label;


	if (ParsedText_Flag!=TRUE)
	{
		n = 0;
		dialogShell = XmCreateDialogShell(
			W_MenuBar, "ParsedWidget", args, n);

  		n = 0;
 		frame = XmCreateFrame(dialogShell, "frame", args, n);

		n = 0;  
		SetTextForm = XmCreateForm(frame, "SetTextForm", args, n);

	/*LABEL  */
     		label = XmStringCreateSimple ("Parsed Text");
     		n = 0;
    		XtSetArg(args[n], XmNlabelString, label);  n++;
    		XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   		XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++; 
   	  	SetTextLabel = XmCreateLabelGadget (SetTextForm, NULL, args, n);
   	  	XtManageChild (SetTextLabel);
     		XmStringFree(label);

	/*SEPARATOR  */
		n = 0;
    		XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    		XtSetArg(args[n], XmNtopWidget, SetTextLabel); n++;
   		XtSetArg(args[n], XmNseparatorType, XmSHADOW_ETCHED_IN); n++;
   	 	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   		XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   	  	Separator = XmCreateSeparatorGadget (SetTextForm, NULL, args, n);
   	  	XtManageChild (Separator);

	/*DOCUMENTNAMELABEL  */
     		label = XmStringCreateSimple ("Documentname");
     		n = 0;
    		XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    		XtSetArg(args[n], XmNtopWidget, Separator); n++;
    		XtSetArg(args[n], XmNlabelString, label);  n++;
    		XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   		XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++; 
   	  	W_DocumentTitle = XmCreateLabelGadget(
				SetTextForm, "DocumentName", args, n);
   	  	XtManageChild (W_DocumentTitle);
     		XmStringFree(label);

	/*SEPARATOR 2  */
		n = 0;
    		XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    		XtSetArg(args[n], XmNtopWidget, W_DocumentTitle); n++;
   		XtSetArg(args[n], XmNseparatorType, XmSHADOW_ETCHED_IN); n++;
   	 	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   		XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   	  	Separator2 = XmCreateSeparatorGadget (SetTextForm, NULL, args, n);
   	  	XtManageChild (Separator2);	

	/*CANCELBUTTONFORM */
    		n=0;
    		XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    		XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    		XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;

    		Cancelbuttonform = XtCreateManagedWidget(
            		"Cancelbuttonform", xmFormWidgetClass, SetTextForm, args, n);  		

	/*HTML  */
	    	n = 0;
	    	XtSetArg(args[n], WbNanchorUnderlines, 1); n++;
	    	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
	    	XtSetArg(args[n], XmNtopWidget, Separator2); n++;
	    	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	   	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
	   	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
	    	XtSetArg(args[n], XmNbottomWidget, Cancelbuttonform); n++;
	
		W_HtmlWidget = XtCreateManagedWidget(
			"HTMLWidget", htmlWidgetClass, SetTextForm, args, n);


	/*CANCELBUTTON  */
		n = 0;
     		XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   		XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   		XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
   		XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++; 

    		Cancel = XtCreateManagedWidget(
            		"Cancel", xmPushButtonWidgetClass, 
			Cancelbuttonform, args, n);  	
		XtAddCallback(Cancel, XmNactivateCallback, SetTextCancel_CB, NULL);		

   		XtManageChild (frame);
		XtManageChild(SetTextForm);
		ParsedText_Flag=TRUE;
	}
	
	documenttitle=GetDocumentTitle();
	if (documenttitle!=NULL)
	{
	     	label = XmStringCreate (documenttitle, XmFONTLIST_DEFAULT_TAG);
	}
	else	label = XmStringCreate ("*NO TITLE*", XmFONTLIST_DEFAULT_TAG);

     	n = 0;
    	XtSetArg(args[n], XmNlabelString, label);  n++;
	XtSetValues(W_DocumentTitle, args, n);
    	XmStringFree(label);
	inhalt = XmTextGetString(W_ScrolledText);

    	HTMLSetText(W_HtmlWidget, inhalt, HTMLHeader, HTMLFooter, 0, NULL, NULL);

	XtFree(inhalt);
	XtFree(documenttitle);
	ScrolledTextSetSensitive();
}

/*----------------------------------------------------------------------*/
/*Copy  */
/*----------------------------------------------------------------------*/

/*COPY FILE TO CLIPBOARD  */
void 
CopyFileToClipboard(time)
Time time;
{
	char *selected_string = XmTextGetSelection (W_ScrolledText);
 
  	long 	      item_id = 0; /* Clipboard item id */
   	long          data_id = 0; /* Clipboard data id */
   	int           status = 0;  /* Clipboard status  */
   	XmString      clip_label;
	XmStringCharSet charset = (XmStringCharSet) XmFONTLIST_DEFAULT_TAG;

/*Copy the selected text to the clipboard  */
   	if (selected_string != NULL) 
	{
  		clip_label = XmStringCreateLtoR ("RELAX", charset);

	/*Start copy to clipboard  */
	  	status = 0;
  		while (status != ClipboardSuccess)
      			status = XmClipboardStartCopy (XtDisplay(W_ScrolledText), 
				XtWindow(W_ScrolledText), clip_label, time, 
				W_ScrolledText, NULL, &item_id);

	/*Move the data to the clipboard  */
  		status = 0;
  		while (status != ClipboardSuccess)
      			status = XmClipboardCopy (XtDisplay(W_ScrolledText), 
				XtWindow(W_ScrolledText), item_id, 
				"STRING", selected_string,
                        	(long)strlen(selected_string)+1, 0, &data_id);
  
	/*End the copy to the clipboard  */
  		status = 0;
  		while (status != ClipboardSuccess)
      			status = XmClipboardEndCopy (XtDisplay(W_ScrolledText),
				XtWindow(W_ScrolledText), item_id);
	}

	XtSetSensitive(W_ButtonPaste, True);
	XtSetSensitive(W_MenuPaste, True);
	ScrolledTextSetSensitive();
}

void 
Menu_Copy(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	CopyFileToClipboard(
		((XmPushButtonCallbackStruct *)call_data)->event->xbutton.time);

	ScrolledTextSetSensitive();
}

/*----------------------------------------------------------------------*/
/*Cut  */
/*----------------------------------------------------------------------*/

void 
Menu_Cut(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
        CopyFileToClipboard(
		((XmPushButtonCallbackStruct *) call_data)->event->xbutton.time);

	/* Call routine to delete primary selection */
        XmTextRemove(W_ScrolledText);         	

	ScrolledTextSetSensitive();
}

/*----------------------------------------------------------------------*/
/*Paste  */
/*----------------------------------------------------------------------*/

void 
Menu_Paste(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{   
   	int            status = 0;            /* Clipboard status        */
   	char          *buffer;                /* Temporary text buffer   */
   	unsigned long  length;                /* Length of buffer        */
  	unsigned long  outlength = 0;         /* Length of bytes copied  */
   	long           private_id = 0;        /* Id of item on clipboard */
   	XmTextPosition cursorPos;             /* Text cursor position    */
   	register int   n;                     /* Arg count               */
   	Arg            args[10];              /* Arg list                */


	EnableUndo();

/* Find the length of the paste item, continue until the length is found */
   	while (status != ClipboardSuccess) 
	{
     		status = XmClipboardInquireLength(XtDisplay(W_ScrolledText),
 			XtWindow(W_ScrolledText), "STRING", &length);

     		if (status == ClipboardNoData)
		{
  			length = 0;
  			break;
     		}
   	}

   	if (length == 0) 
	{
  		fprintf(stderr, "Warning: paste failed, no items to paste.\n");
  		return;
   	}

/* Malloc to necessary space */
   	buffer = XtMalloc(length);

   	status = XmClipboardRetrieve (XtDisplay(W_ScrolledText),
 		XtWindow(W_ScrolledText), "STRING",
         	buffer, length, &outlength, &private_id);

/* Error in pasting */
   	if (status != ClipboardSuccess) 
	{
  		fprintf(stderr, "Warning: paste failed, status = %d\n", status);  
  		return;
   	}

/* Get cursor position for pasting */
   	XtSetArg(args[0], XmNcursorPosition, &cursorPos);
   	XtGetValues(W_ScrolledText, args, 1);
      
/* Add new text */
   	XmTextReplace(W_ScrolledText, cursorPos, cursorPos, buffer);

	ScrolledTextSetSensitive();
}

/*----------------------------------------------------------------------*/
/*Delete  */
/*----------------------------------------------------------------------*/

void 
Menu_Delete(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	EnableUndo();

	/* Call routine to delete primary selection */
        XmTextRemove(W_ScrolledText);         	

	ScrolledTextSetSensitive();
}
/*----------------------------------------------------------------------*/
/*Enable Undo */
/*----------------------------------------------------------------------*/

void 
EnableUndo()
{
	Arg           args[10];

	XtSetArg(args[0], XmNcursorPosition, &UndoCursorPos);
	XtGetValues(W_ScrolledText, args, 1);

	if( strlen(UndoBuffer) > 0 )
		XtFree(UndoBuffer);
  	UndoBuffer = XmTextGetString(W_ScrolledText);

	ScrolledTextSetSensitive();
}


/*----------------------------------------------------------------------*/
/*Undo  */
/*----------------------------------------------------------------------*/

void 
Menu_Undo(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	Arg           args[10];
    	int           n;

/* Enable Redo */
	XtSetArg(args[0], XmNcursorPosition, &RedoCursorPos);
	XtGetValues(W_ScrolledText, args, 1);
 	RedoBuffer = XmTextGetString(W_ScrolledText);

/* Undo */
	XmTextSetString(W_ScrolledText, UndoBuffer);
 	XtSetArg(args[0], XmNcursorPosition, UndoCursorPos);
   	XtSetValues(W_ScrolledText, args, 1);

/* Enable Redo */
	if( strlen(UndoBuffer) > 0 )
		XtFree(UndoBuffer);
	UndoBuffer = malloc( strlen(RedoBuffer) + 1 );
  	strcpy(UndoBuffer, RedoBuffer);	
	if( RedoBuffer != NULL && strlen(RedoBuffer) > 0 )
		XtFree(RedoBuffer);
	RedoBuffer = NULL;
 	UndoCursorPos = RedoCursorPos;

	ScrolledTextSetSensitive();
}
