/*	
 *	relax - an editor for creating HTML documents
 *	
 *	Copyright (C) 1994 by the Department of Computer Science at the TUM
 *	(Munich University of Technology, Germany)
 *	
 *	Authors: Bernhard Blaette, Michael Tiefenbacher
 *	
 *	This program is partly based on the NCSA Mosaic software.
 *	NCSA Mosaic software, both binary and source (hereafter, Software) is
 *	copyrighted by The Board of Trustees of the University of Illinois
 *	(UI), and ownership remains with the UI.
 *	The editor "relax" uses the 'htmlWidgetClass' and the 'HTMLSetText' 
 *	procedure of the 'HTML.h' include file for parsing the edited text 
 *	into a Mosaic formatted style. Moreover, 'HTMLamp.h' is included for
 *	the special character popup. 
 *	Certainly, "relax" is not the original NCSA Mosaic but uses some of
 *	its Software for such derivative work.
 *	
 *	Permission to use, copy, modify, and distribute this software and its 
 *	documentation for any purpose and without fee is hereby granted,
 *	provided that this comment and the authors' names are retained and
 *	appear in all copies and that both that copyright notice and this 
 *	permission notice appear in supporting documentation.
 *	
 *	This software is provided 'as is' without express or implied warranty.
 *	In no event shall the authors or the TUM be liable for errors contained 
 *	in the software, or for direct, indirect, special, incidental or 
 *	consequential damages with the use of the software.
 *	
 */
#include <stdio.h>
#include <sys/stat.h>

#include <relax_widget-stuff.h>
#include <relax_global-defines.h>

/*----------------------------------------------------------------------------*/
char 	NullChar='\0';
char*  	CheckBuffer;
char* 	HTMLHeader=NULL;
char* 	HTMLFooter=NULL;
char* 	TextFileName=NULL;
Bool 	Save_Flag=FALSE;
Bool	Load_Flag=FALSE;

void	SaveLists();
void	LoadLists();


/*----------------------------------------------------------------------------*/
/*Quit  */
/*----------------------------------------------------------------------------*/

void
Quit(widget)
Widget	widget;
{
	SaveLists();

    	XtDestroyApplicationContext(XtWidgetToApplicationContext(widget));
    	exit(0);
}

/*----------------------------------------------------------------------------*/
/*FileInfo*/
/*----------------------------------------------------------------------------*/

Bool
FileInfo(filename, filesize)
char*	filename;
int*	filesize;
{
	struct stat 	finfo;

	if( stat(filename, &finfo) != 0 )
        {
		printf("No such file: %s\n", filename);
		
		*filesize = -1;
		return(FALSE);
	}
	else
	{	*filesize = finfo.st_size;
		return(TRUE);
	}
}


/*----------------------------------------------------------------------------*/
/*Read-/WriteFile*/
/*----------------------------------------------------------------------------*/
char* 
ReadFile(filename)
char* filename;
{
        FILE *f;
        long int byte_anzahl;
	int size=1;     
	char *inhalt;

        if(( f=fopen(filename,"r") ) == NULL)
        {
                /*File does not exist!  */
		printf("File %s is not readable!\n", filename);
        	return(NULL);
        }
        else
        {
		rewind(f);
		fseek(f, 0L, SEEK_END);
		byte_anzahl = ftell(f);
		rewind(f);
			
		inhalt = malloc(size*(byte_anzahl+1));
		inhalt[byte_anzahl]='\0';

                if( fread(inhalt,1,byte_anzahl,f) != byte_anzahl)
                {
                        printf("Incorrect file length!\n");
                        free(inhalt);
                        inhalt=NULL;
                }

		inhalt[byte_anzahl-1]='\0';

                if(fclose(f)!=0)
                {
                        printf("Close file error!\n");
                        free(inhalt);
                        inhalt=NULL;
                }
		return(inhalt);
        }
}

Bool WriteFile(filename, text)
char* filename;
char* text;
{
	FILE *f;
	int ZeichAnzahl;
	Bool ok=TRUE;

	if (( f=fopen(filename,"w")) == NULL)
	{
		printf("Open file error %s!\n", filename);
		return(FALSE);
	}

	ZeichAnzahl = strlen(text)+1;
	text[ZeichAnzahl-1] = '\012';

	if( fwrite(text,1,ZeichAnzahl,f) != ZeichAnzahl )
	{
		printf("Write file error %s!\n", filename);
		ok=FALSE;
	}

	text[ZeichAnzahl-1] = '\0';
	if(fclose(f)!=0)
	{
		printf("Close file error %s!\n", filename);
		ok=FALSE;
	}
	return(ok);
}

/*----------------------------------------------------------------------------*/
/*Set File Label  */
/*----------------------------------------------------------------------------*/

void
SetFileNameLabel(label)
char* label;
{
    	XmString      label_string;
    	Arg           args[20];
	int n;

    	label_string = XmStringCreateSimple(label);

    	n = 0;
     	XtSetArg(args[n], XmNlabelString,label_string); n++;
     	XtSetValues(W_FileLabel, args, n);
    	XmStringFree(label_string);
}

/*----------------------------------------------------------------------*/
/*Save Lists  */
/*----------------------------------------------------------------------*/

void 
SaveLists()
{
	char*		links= (char*) malloc( 1100*sizeof(char) );
	char*		images=(char*) malloc( 1100*sizeof(char) );
	char		filetext[10000];
	Cardinal	h=0;
	int 		len=0;		

	/*Save LINK - List  */
	filetext[0] = '\0';
	while (h < Link_Counter)
	{	
		XmStringGetLtoR(Link_List[h], XmFONTLIST_DEFAULT_TAG, &links);
		strcat (filetext, links);
		strcat (filetext, "\n");
		h++;
	}
	len = strlen( filetext )-1;
	if( len >= 0 )
	{	filetext[len] = '\0';   /* last '\n' --> '\0' */
	}
	WriteFile(FileNameLinkHist, filetext, FALSE);

	/*Save IMAGE - List  */
	h=0;
	filetext[0] = '\0';
	while (h < Image_Counter)
	{	
		XmStringGetLtoR(Image_List[h], XmFONTLIST_DEFAULT_TAG, &images);
		strcat (filetext, images);
		strcat (filetext, "\n");
		h++;
	}
	len = strlen( filetext )-1;
	if( len >= 0 )
	{	filetext[len] = '\0';   /* last '\n' --> '\0' */
	}
	WriteFile(FileNameImageHist, filetext, FALSE);

	XtFree(links);
	XtFree(images);
}

/*----------------------------------------------------------------------*/
/*Load-Link/Image-List  */
/*----------------------------------------------------------------------*/

void 
LoadLists(list, filename_list, c)
XmString *list;	
char*	 filename_list;
Cardinal *c;	
{
	char		*filetext;
	char		inhalt[1000];
	Cardinal	h=0, i, j=0;
	XmString	entry;

	if ((*c) != 0)
	{
		for (i=0; i<(*c); i++)
		{
			XmStringFree(list[i]);
		}
	}		

 	if( (filetext= ReadFile(filename_list)) == NULL )
	{	return;
	}

	*c=0;          /*list counter set to Zero !  */

	while( filetext[h] != '\0' )
	{
		j=0;
		while( filetext[h] != '\0' && filetext[h] != '\n' )
		{
			inhalt[j] = filetext[h];
			h++;
			j++;
		}
		if( j > 0 )
		{
			inhalt[j] = '\0';
 			entry = XmStringCreateLtoR(
				inhalt, XmFONTLIST_DEFAULT_TAG);
			list[*c] = entry;

			(*c)++;   /*Increase list counter  */
		}
		if( filetext[h] != '\0' )
		{
			h++;
		}		
	}
 	XtFree(filetext);
}

/*----------------------------------------------------------------------*/
/*SaveDialog-Popup  */
/*----------------------------------------------------------------------*/

void 
SaveDialogOk_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{	
	Widget ok, savedialogbox, dialogtext;

	ok = (Widget) widget;
	dialogtext = (Widget) client_data;
	savedialogbox = XtParent(ok);

	if (TextFileName!=NULL)
		XtFree(TextFileName);
	TextFileName = XmTextFieldGetString(dialogtext);

	if( strlen(CheckBuffer) > 0 )
		XtFree(CheckBuffer);
	CheckBuffer = XmTextGetString(W_ScrolledText);
 
	SaveLists();

    	if (WriteFile(TextFileName, CheckBuffer))
	{ 
		SetFileNameLabel(TextFileName);
		XtDestroyWidget(XtParent(XtParent(XtParent(savedialogbox))));
		Save_Flag=FALSE;
    	}
}


void 
SaveDialogCancel_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{	
	Widget savedialogbox;

	savedialogbox = (Widget)client_data;

	XtDestroyWidget(savedialogbox);

	Save_Flag=FALSE;
}

void 
Menu_SaveDialog(widget, client_data, call_data)
Widget	widget;	
XtPointer client_data, call_data;
{
	Widget savedialogbox, help;	
	Widget ok, cancel, dialogtext;
	Widget form, separator, labeltext;
	Widget frame, dialogShell;
 	XmString label;
 	Arg args[20];
    	int n;

	if (Save_Flag==TRUE) return;

     	n = 0;
 	dialogShell = XmCreateDialogShell(W_MenuBar, "SaveDialog", args, n);

	n = 0;
 	frame = XmCreateFrame(dialogShell, "frame", args, n);

	n = 0;
	form = XmCreateForm(frame, "fileform", args, n);

	/*LABEL  */
   	label = XmStringCreateSimple ("SAVE");
     	n = 0;
    	XtSetArg(args[n], XmNlabelString, label);  n++;
    	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
    	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++; 
   	labeltext = XmCreateLabelGadget (form, NULL, args, n);
   	XtManageChild (labeltext);
     	XmStringFree(label);

	/*SEPARATOR  */
	n = 0;
    	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    	XtSetArg(args[n], XmNtopWidget, labeltext); n++;
   	XtSetArg(args[n], XmNseparatorType, XmSHADOW_ETCHED_IN); n++;
   	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   	separator = XmCreateSeparatorGadget (form, NULL, args, n);
   	XtManageChild (separator);

	n = 0;
    	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    	XtSetArg(args[n], XmNtopWidget, separator); n++;
   	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;

   	savedialogbox =  XmCreateFileSelectionBox(
            form, "SaveDialog-Box", args, n);    

	help = XmFileSelectionBoxGetChild(
		savedialogbox, XmDIALOG_HELP_BUTTON);
	ok = XmFileSelectionBoxGetChild(
		savedialogbox, XmDIALOG_OK_BUTTON);
	cancel = XmFileSelectionBoxGetChild(
		savedialogbox, XmDIALOG_CANCEL_BUTTON);
	dialogtext = XmFileSelectionBoxGetChild(
	    	savedialogbox, XmDIALOG_TEXT);

	XtUnmanageChild(help);

	XtAddCallback(ok, XmNactivateCallback, SaveDialogOk_CB, 
		(XtPointer) dialogtext);
	XtAddCallback(cancel, XmNactivateCallback, SaveDialogCancel_CB, 
		(XtPointer) dialogShell);

	Save_Flag=TRUE;

     	XtManageChild (savedialogbox);

   	XtManageChild (frame);
   	XtManageChild (form);

 	ScrolledTextSetSensitive();
}


void	
Menu_SaveText(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	if (TextFileName!=NULL)
	{
		if( strlen(CheckBuffer) > 0 )
			XtFree(CheckBuffer);
		CheckBuffer = XmTextGetString(W_ScrolledText);
		WriteFile(TextFileName, CheckBuffer);

		SaveLists();
	}
	else Menu_SaveDialog(widget, client_data, call_data);
}

/*----------------------------------------------------------------------*/
/*LoadDialog-Popup  */
/*----------------------------------------------------------------------*/

void 
LoadDialogOk_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{	
	Widget ok, loaddialogbox, dialogtext;
	char* inhalt;

	ok = (Widget) widget;
	dialogtext = (Widget) client_data;
	loaddialogbox = XtParent(ok);

	if (TextFileName!=NULL)
		XtFree(TextFileName);
	TextFileName = XmTextFieldGetString(dialogtext);

    	inhalt = ReadFile(TextFileName);
    	if (inhalt != NULL)
	{ 
		SetFileNameLabel(TextFileName);

         	if( strlen(CheckBuffer) > 0 )
         	        XtFree(CheckBuffer);
		CheckBuffer = malloc( strlen(inhalt)+1 );
		(void) strcpy(CheckBuffer,inhalt);

		XmTextSetString(W_ScrolledText, CheckBuffer);
		free(inhalt);
		XtDestroyWidget(XtParent(XtParent(XtParent(loaddialogbox))));
		Load_Flag=FALSE;
    	}
}


void 
LoadDialogCancel_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{	
	Widget loaddialogbox;

	loaddialogbox = (Widget) client_data;

	XtDestroyWidget(loaddialogbox);

	Load_Flag=FALSE;
}

void
LoadDialog(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{	
	Widget loaddialogbox, help, ok, cancel, dialogtext;
	Widget form, separator, labeltext, frame, dialogShell;
 	XmString label;
 	Arg args[20];
    	int n;
	
	if (Load_Flag==TRUE) return;
 

     	n = 0;
	dialogShell = XmCreateDialogShell(W_MenuBar, "LoadDialog", args, n);

  	n = 0;
 	frame = XmCreateFrame(dialogShell, "frame", args, n);

     	n = 0;
	form = XmCreateForm(frame, "fileform", args, n);

	/*LABEL  */
   	label = XmStringCreateSimple ("LOAD");
     	n = 0;
    	XtSetArg(args[n], XmNlabelString, label);  n++;
    	XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
    	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++; 
   	labeltext = XmCreateLabelGadget (form, NULL, args, n);
   	XtManageChild (labeltext);
     	XmStringFree(label);

	/*SEPARATOR  */
	n = 0;
    	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    	XtSetArg(args[n], XmNtopWidget, labeltext); n++;
   	XtSetArg(args[n], XmNseparatorType, XmSHADOW_ETCHED_IN); n++;
   	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   	separator = XmCreateSeparatorGadget (form, NULL, args, n);
   	XtManageChild (separator);

	n = 0;
    	XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    	XtSetArg(args[n], XmNtopWidget, separator); n++;
   	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
   	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
   	XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;

   	loaddialogbox =  XmCreateFileSelectionBox(
            form, "LoadDialog-Box", args, n);    

	help = XmFileSelectionBoxGetChild (loaddialogbox, XmDIALOG_HELP_BUTTON);
	ok = XmFileSelectionBoxGetChild (loaddialogbox, XmDIALOG_OK_BUTTON);
	cancel = XmFileSelectionBoxGetChild (
		loaddialogbox, XmDIALOG_CANCEL_BUTTON);
	dialogtext = XmFileSelectionBoxGetChild (loaddialogbox, XmDIALOG_TEXT);

	XtUnmanageChild(help);

	XtAddCallback(ok, XmNactivateCallback, LoadDialogOk_CB, 
		(XtPointer) dialogtext);
	XtAddCallback(cancel, XmNactivateCallback, LoadDialogCancel_CB,
		(XtPointer) dialogShell);

	Load_Flag=TRUE;
     	XtManageChild (loaddialogbox);

   	XtManageChild (frame);
   	XtManageChild (form);
}

/*----------------------------------------------------------------------*/
/*New  */
/*----------------------------------------------------------------------*/
 
void 
New()
{
	XmTextSetString (W_ScrolledText, "");
	CheckBuffer = &NullChar;
   
	/* Free the file name */
    	if (TextFileName != NULL) 
	{
        	XtFree(TextFileName);
        	TextFileName = NULL;
	}
	SetFileNameLabel("*unknown*");
}

/*----------------------------------------------------------------------*/
/*Warning Dialog - Unsaved Changes!  */
/*----------------------------------------------------------------------*/

void 
WarningBoxDiscard_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{	
	char* Callingwidget;
	
	Callingwidget = (XtPointer)client_data;
	
	if (strcmp((char*)Callingwidget, "quit")==0)
	{
		Quit(widget);
	}
	else if (strcmp((char*)Callingwidget, "new")==0)
	{
		New();
	}	
	else if (strcmp((char*)Callingwidget, "load")==0)
	{
		LoadDialog(widget, client_data, call_data);
	}
}

void 
WarningBoxCancel_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{		
	Widget cancel, warningdialogbox;
	
	cancel = (Widget) widget;
	warningdialogbox = XtParent(cancel);

	XtDestroyWidget(warningdialogbox);
}

void 
WarningDialog(Callingwidget)
char* Callingwidget;		
{
	Arg  		args[10];
     	Cardinal  	n;
     	Widget  	warningbox, ok;
       	
	n = 0;
   	XtSetArg (args[n], XmNmessageString, 
			XmStringCreateLtoR ("There are unsaved changes !",
       			XmFONTLIST_DEFAULT_TAG)); n++;
    	XtSetArg (args[n], XmNcancelLabelString, XmStringCreateLtoR ("Discard",
       			XmFONTLIST_DEFAULT_TAG)); n++;
   	XtSetArg (args[n], XmNhelpLabelString, XmStringCreateLtoR ("Cancel",
       			XmFONTLIST_DEFAULT_TAG)); n++;

     	warningbox = XmCreateWarningDialog (W_MenuBar, "warningbox", args, n);

	ok = XmMessageBoxGetChild(warningbox, XmDIALOG_OK_BUTTON);
	XtUnmanageChild(ok);
  
	XtAddCallback (warningbox, XmNcancelCallback,
			 WarningBoxDiscard_CB, Callingwidget);
	XtAddCallback (warningbox, XmNhelpCallback, WarningBoxCancel_CB, NULL);
     	
	XtManageChild (warningbox);
}

/*----------------------------------------------------------------------*/
/*LoadCheck  */
/*----------------------------------------------------------------------*/

void 
Menu_LoadCheck(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	char* inhalt;
	char* Callingwidget;

	Callingwidget = "load";
   	inhalt = XmTextGetString(W_ScrolledText);

	if (strcmp(CheckBuffer, inhalt)==0)
	{ 	
		LoadDialog(widget, client_data, call_data);
	}
	else
	{
		WarningDialog(Callingwidget);
	}
	XtFree(inhalt);
}

/*----------------------------------------------------------------------*/
/*NewCheck  */
/*----------------------------------------------------------------------*/

void 
Menu_NewCheck(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	char* inhalt;
	char* Callingwidget;

	Callingwidget = "new";
   	inhalt = XmTextGetString(W_ScrolledText);

	if (strcmp(CheckBuffer, inhalt)==0)
	{ 	
		New();
	}
	else
	{
		WarningDialog(Callingwidget);
	}	
	XtFree(inhalt);
}

/*----------------------------------------------------------------------*/
/*QuitCheck  */
/*----------------------------------------------------------------------*/

void 
Menu_QuitCheck(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	char* inhalt;
	char* Callingwidget;

	Callingwidget = "quit";
   	inhalt = XmTextGetString(W_ScrolledText);

	if (strcmp(CheckBuffer, inhalt)==0)
	{
		Quit(widget);
	}
	else
	{ 
		WarningDialog(Callingwidget);
	}	
	XtFree(inhalt);
}


