/*	
 *	relax - an editor for creating HTML documents
 *	
 *	Copyright (C) 1994 by the Department of Computer Science at the TUM
 *	(Munich University of Technology, Germany)
 *	
 *	Authors: Bernhard Blaette, Michael Tiefenbacher
 *	
 *	This program is partly based on the NCSA Mosaic software.
 *	NCSA Mosaic software, both binary and source (hereafter, Software) is
 *	copyrighted by The Board of Trustees of the University of Illinois
 *	(UI), and ownership remains with the UI.
 *	The editor "relax" uses the 'htmlWidgetClass' and the 'HTMLSetText' 
 *	procedure of the 'HTML.h' include file for parsing the edited text 
 *	into a Mosaic formatted style. Moreover, 'HTMLamp.h' is included for
 *	the special character popup. 
 *	Certainly, "relax" is not the original NCSA Mosaic but uses some of
 *	its Software for such derivative work.
 *	
 *	Permission to use, copy, modify, and distribute this software and its 
 *	documentation for any purpose and without fee is hereby granted,
 *	provided that this comment and the authors' names are retained and
 *	appear in all copies and that both that copyright notice and this 
 *	permission notice appear in supporting documentation.
 *	
 *	This software is provided 'as is' without express or implied warranty.
 *	In no event shall the authors or the TUM be liable for errors contained 
 *	in the software, or for direct, indirect, special, incidental or 
 *	consequential damages with the use of the software.
 *	
 */
#include <stdio.h>

#include <relax_widget-stuff.h>
#include <relax_global-defines.h>

/*----------------------------------------------------------------------------*/
Widget		W_AlignementLabel;
Widget		W_AliDialog;

Cardinal	Link_Counter=0;
Cardinal	Anchor_Counter=0;
Cardinal	Image_Counter=0;
Cardinal	AnchorLinkImage=0;

Bool 		TopAlignement_Flag=FALSE;
Bool		AnchorDialog_Flag = FALSE;
Bool		LinkDialog_Flag = FALSE;
Bool		ImageDialog_Flag = FALSE;

XmString 	Link_List[200];
XmString 	Anchor_List[200];
XmString 	Image_List[200];


/*----------------------------------------------------------------------*/
/*Set Label - Bottom or Top Alignment  */
/*----------------------------------------------------------------------*/

void
SetLabel(label)
char* label;
{
    	XmString      label_string;
    	Arg           args[10];
	int 	      n;

    	label_string = XmStringCreate(label, XmFONTLIST_DEFAULT_TAG);

    	n = 0;
     	XtSetArg(args[n], XmNlabelString, label_string); n++;
	XtSetValues(W_AlignementLabel, args, n);
}

/*----------------------------------------------------------------------*/
/*Add Item to list  */
/*----------------------------------------------------------------------*/

void 
AddItemToList(item, list, var)
char		*item;
XmString 	*list;
Cardinal 	*var;
{
     	Arg  		args[10];
     	int  		n, y;
	XmString	itemString;
	
	itemString = XmStringCreate(item, XmFONTLIST_DEFAULT_TAG);  	

	for (y = 0; y < *var; y++)
	{	
		if (XmStringCompare((XmString)itemString, 		
				(XmString)list[y])!=0)
		{	
			printf("Item already exists!\n"); 
			return;
		}	
	}

	list[*var] = (XmString) XmStringCreateLtoR(
			item, XmFONTLIST_DEFAULT_TAG);
	(*var)++;
	
	n = 0;
      	XtSetArg(args[n], XmNlistItems, list); n++;
   	XtSetArg(args[n], XmNlistItemCount, *var); n++;
	XtSetValues(W_AliDialog, args, n);	

	XmStringFree(itemString);
}

/*----------------------------------------------------------------------*/
/*Delete Item from list  */
/*----------------------------------------------------------------------*/

void 
DeleteItemFromList(item, list, var)
char		*item;
XmString 	*list;
Cardinal 	*var;
{
    	Arg  		args[10];
     	int  		n, y, r;
	XmString	delitemString;

	r = 0;
	delitemString = XmStringCreate(item, XmFONTLIST_DEFAULT_TAG);

	for (y = 0; y < *var; y++)
	{	
		/*Attension: XmStringCompare is very differnt to strcmp!  */
		if (XmStringCompare((XmString)delitemString, 
				(XmString)list[y])!=0)
		{	
			*var = (*var-1);

			/*Delete Item from List  */
			while (y != *var)
			{				
				r = y+1;
				list[y] = list[r]; 
				y++;
			}
	
			/*Set new list items  */		
			n = 0;
			XtSetArg(args[n], XmNlistItems, list); n++;
   			XtSetArg(args[n], XmNlistItemCount, *var); n++;
			XtSetValues(W_AliDialog, args, n);
		}
	}

	XmStringFree(delitemString); 	
}

/*----------------------------------------------------------------------*/
/*Add and Delete LinkImage Callback  */
/*----------------------------------------------------------------------*/

void 
LinkImageAdd_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{	
	Widget 	dialogtext;
	char* 	additem;

	dialogtext = (Widget) client_data; 
	additem = XmTextFieldGetString(dialogtext);

	if (strcmp(additem, "")==0)
	{
		printf("empty items are not added\n");
		return;
	}
	
	if (AnchorLinkImage == 2)
	{
		AddItemToList(additem, Link_List, &Link_Counter);	
	}
	else
	{
		AddItemToList(additem, Image_List, &Image_Counter);
	}
	XtFree(additem);
}

void 
LinkImageDelete_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{	
	Widget 		dialogtext;
	char* 		delitem;

	dialogtext = (Widget) client_data;
	delitem = XmTextFieldGetString(dialogtext);
	
	if (AnchorLinkImage == 2)
	{
		DeleteItemFromList(delitem, Link_List, &Link_Counter);	
	}
	else
	{
		DeleteItemFromList(delitem, Image_List, &Image_Counter);
	}
	XtFree(delitem);
}

/*----------------------------------------------------------------------*/
/*Change Alignment Callback  */
/*----------------------------------------------------------------------*/

void 
ChangeAlign_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
    	char*         label_string;

	TopAlignement_Flag = 	!(TopAlignement_Flag);	

	if (TopAlignement_Flag == True) 
	{
		label_string = "Top Alignment";
	}
	else
	{
		label_string = "Bottom Alignment";	
	}

	SetLabel(label_string);	
}

/*----------------------------------------------------------------------*/
/*OK Callbacks  */
/*----------------------------------------------------------------------*/

void 
LinkOk_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	Widget 	dialogtext, dialog, ok;
	char	*link;
	char	*buffer1;               
 	char    *buffer2;
	char    buffer3[100];

	ok = (Widget) widget;
	dialog = XtParent(XtParent(XtParent(ok)));
	dialogtext = (Widget) client_data;
	link = XmTextFieldGetString(dialogtext);
	LinkDialog_Flag = FALSE;

	buffer1 = "<A HREF=\"";
	buffer2 = "\">";
	strcpy (buffer3, buffer1);
	strcat (buffer3, link);
	strcat (buffer3, buffer2);
	buffer2 = "</A>";

	XtDestroyWidget(dialog);
	XtFree(link);

	InsertTags(buffer3, buffer2);
}

void 
AnchorOk_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	Widget 	dialogtext, dialog, ok;
	char* 	anchor;
  	char	*buffer1;               
 	char    *buffer2;
	char    buffer3[100];

	ok = (Widget) widget;
	dialog = XtParent(XtParent(XtParent(ok)));
	dialogtext = (Widget) client_data;
	anchor = XmTextFieldGetString(dialogtext);
	AnchorDialog_Flag = FALSE;

	/*Additional insert in history list  */
	if (strcmp(anchor, "")==0)
	{
		printf("empty anchors are not added !\n");
		return;
	}
	
	AddItemToList(anchor, Anchor_List, &Anchor_Counter);			

	buffer1 = "<A HREF=\"#";
	buffer2 = "\">";
	strcpy (buffer3, buffer1);
	strcat (buffer3, anchor);
	strcat (buffer3, buffer2);
	buffer2 = "</A>";

	XtDestroyWidget(dialog);
	XtFree(anchor);

	InsertTags(buffer3, buffer2);
}

void 
ImageOk_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	Widget 	dialogtext, dialog, ok;
	char* 	image;
	char	*buffer1;               
 	char    *buffer2;
 	char    *buffer4;
	char    buffer3[100];

	ok = (Widget) widget;
	dialog = XtParent(XtParent(XtParent(ok)));
	dialogtext = (Widget) client_data;
	image = XmTextFieldGetString(dialogtext);
	ImageDialog_Flag = FALSE;

	if (TopAlignement_Flag == True)	
	{
		buffer4 = "ALIGN=top SRC=\"";
	}
	else
	{
		buffer4 = "SRC=\"";
	}

	buffer1 = "<IMG ";
	buffer2 = "\">";
	strcpy (buffer3, buffer1);
	strcat (buffer3, buffer4);
	strcat (buffer3, image);
	strcat (buffer3, buffer2);

	XtDestroyWidget(dialog);
	XtFree(image);

	InsertText(buffer3);
}

/*----------------------------------------------------------------------*/
/*CANCEL Callback  */
/*----------------------------------------------------------------------*/

void 
AnchorLinkCancel_CB(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{	
	Widget cancel, dialog;
	Bool*  flag;
	
	flag = (Bool*) client_data;
	*flag = FALSE;

	cancel = (Widget) widget;
	dialog = XtParent(XtParent(XtParent(cancel)));

	XtDestroyWidget(dialog);
}


/*----------------------------------------------------------------------*/
/*Anchor - Link - Image - Dialog  */
/*----------------------------------------------------------------------*/

void 
AliDialog(widget, flagadr)
Widget	widget;	
Bool*	flagadr;
{
     	Arg  		args[10];
     	int  		n;
     	Widget		help, add, delete, ok, cancel, apply, dialogtext;
   	Widget		xtra_label, xtraform, separator, separator2; 
	Widget		changebutton, frame, dialogShell;
	XmString	alignlabelstring;
	XmString	label, chbuttonlabel1;

	if ((AnchorLinkImage == 2)&&(LinkDialog_Flag == TRUE))
	{
		printf("Linkdialog is already created !\n"); 	
		return;
	}
	else if ((AnchorLinkImage == 1)&&(AnchorDialog_Flag == TRUE))
	{
		printf("Anchordialog is already created !\n"); 	
		return;
	}
	else if ((AnchorLinkImage == 3)&&(ImageDialog_Flag == TRUE))
	{
		printf("Imagedialog is already created !\n"); 	
		return;
	}

/*SELECTION DIALOG  */
	n = 0;
	XtSetArg(args[n], XmNallowShellResize, FALSE); n++;
	dialogShell = XmCreateDialogShell(widget, "ALIDialog", args, n);

  	n = 0;
	XtSetArg(args[n], XmNwidth, 500); n++;
     	XtSetArg(args[n], XmNheight, 400); n++;
	frame = XmCreateFrame(dialogShell, "frame", args, n);

	n = 0;
	
	if (AnchorLinkImage == 2)
	{
		if (Link_Counter != 0)
		{
			XtSetArg(args[n], XmNlistItems, Link_List); n++;
 	  		XtSetArg(args[n], XmNlistItemCount, Link_Counter); n++;
		}
	}
	else if (AnchorLinkImage == 1)
	{	
		if (Anchor_Counter != 0)
		{			
			XtSetArg(args[n], XmNlistItems, Anchor_List); n++;
   			XtSetArg(args[n], XmNlistItemCount, Anchor_Counter); n++;
		}
	}
	else 
	{
		TopAlignement_Flag=FALSE;

		if (Image_Counter != 0)
		{
			XtSetArg(args[n], XmNlistItems, Image_List); n++;
   			XtSetArg(args[n], XmNlistItemCount, Image_Counter); n++;
		}
	}

	XtSetArg(args[n], XmNwidth, 500); n++;
   	XtSetArg(args[n], XmNheight, 400); n++;

	W_AliDialog = XmCreateSelectionBox (frame, "ALIDialog", args, n);

	/*Only for Link and Image  */
	if (AnchorLinkImage != 1)
	{
	/*EXTRA FORM  */
 		n = 0;
 		xtraform = XtCreateManagedWidget(
			"xtraform", xmFormWidgetClass, W_AliDialog, args, n);  		

	/*Add - Button  */  	
		n = 0;
        	XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
     		XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;        										
		XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
		XtSetArg(args[n], XmNrightPosition, 50); n++;
		XtSetArg(args[n], XmNleftOffset, 3); n++;
		XtSetArg(args[n], XmNtopOffset, 3); n++; 
	
		add = XmCreatePushButton(xtraform, "Add", args, n);
     		XtManageChild(add);

	/*Delete - Button  */  	
		n = 0;
  		XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
     		XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++; 
     		XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++; 
		XtSetArg(args[n], XmNrightOffset, 3); n++; 
		XtSetArg(args[n], XmNtopOffset, 3); n++; 
		XtSetArg(args[n], XmNleftWidget, add); n++;	

		delete = XmCreatePushButton(xtraform, "Delete", args, n);
     		XtManageChild(delete);

	/*SEPARATOR  */
  		n = 0;
     		XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++; 
 		XtSetArg(args[n], XmNtopWidget, delete); n++;	
		XtSetArg(args[n], XmNtopOffset, 3); n++; 
		XtSetArg(args[n], XmNleftOffset, 3); n++; 
   		XtSetArg(args[n], XmNrightOffset, 3); n++; 	
   		XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	    	XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++; 
	
		separator = XmCreateSeparator(xtraform, "separator", args, n);

	    	XtManageChild(separator);

		/*Only for Image  */
		if (AnchorLinkImage == 3)    
		{
       		 /*CHANGE ALIGNMENT BUTTON  */
			chbuttonlabel1 = XmStringCreateLtoR(
				"Change Alignment", XmFONTLIST_DEFAULT_TAG);
			n = 0;
			XtSetArg(args[n], XmNlabelString, chbuttonlabel1); n++;
			XtSetArg(args[n], XmNtopOffset, 6); n++; 
			XtSetArg(args[n], XmNleftOffset, 3); n++; 
			XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++; 
 			XtSetArg(args[n], XmNtopWidget, separator); n++;
			XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++; 
			changebutton = XmCreatePushButton(
				xtraform, "changebutton", args, n);
       			XtManageChild(changebutton);
			XmStringFree (chbuttonlabel1);

		/*CHANGE ALIGN LABEL  */
			alignlabelstring = XmStringCreateLtoR(
				"Bottom Alignment (default)", XmFONTLIST_DEFAULT_TAG);

			n = 0;
			XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
			XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++; 
 			XtSetArg(args[n], XmNleftWidget, changebutton); n++; 
			XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++; 
 			XtSetArg(args[n], XmNtopWidget, separator); n++;
			XtSetArg(args[n], XmNtopOffset, 9); n++; 
   			XtSetArg(args[n], XmNlabelString, alignlabelstring); n++;
	
			W_AlignementLabel = XmCreateLabelGadget(
					xtraform, "W_AlignementLabel", args, n);

	    		XtManageChild(W_AlignementLabel);
  			XmStringFree (alignlabelstring);
		
		/*SEPARATOR2  */
  			n = 0;
     			XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++; 
 			XtSetArg(args[n], XmNtopWidget,	changebutton); n++;	
			XtSetArg(args[n], XmNtopOffset, 3); n++; 
			XtSetArg(args[n], XmNleftOffset, 3); n++; 	
   			XtSetArg(args[n], XmNrightOffset, 3); n++; 	
   			XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
	    		XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++; 
	
			separator2 = XmCreateSeparator(xtraform, "separator2", args, n);

	    		XtManageChild(separator2);

		/*EXTRA LABEL  */
	   		label = XmStringCreateSimple (
				"Sample-Image:  filename.ext  (GIF, TIFF, JPEG, RGB, or HDF)");
			n = 0;
    			XtSetArg(args[n], XmNlabelString, label);  n++;
    			XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    			XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++; 
     			XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++; 
  			XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++; 
 			XtSetArg(args[n], XmNtopWidget, separator2); n++;
			XtSetArg(args[n], XmNtopOffset, 5); n++; 	
			XtSetArg(args[n], XmNbottomOffset, 5); n++; 	
		}
 		/*Only for Link  */
		else   
		{
		/*EXTRA LABEL  */ 
	   		label = XmStringCreateSimple (
				"Sample-Link:  scheme://host.domain[:port]/path/filename[#anchorname]");

     			n = 0;
    			XtSetArg(args[n], XmNlabelString, label);  n++;
    			XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    			XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++; 
     			XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++; 
  			XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++; 
 			XtSetArg(args[n], XmNtopWidget, separator); n++;
			XtSetArg(args[n], XmNtopOffset, 8); n++; 
			XtSetArg(args[n], XmNbottomOffset, 5); n++; 	
		}

		xtra_label = XmCreateLabelGadget(xtraform, "extralabel", args, n);

	    	XtManageChild(xtra_label);
		XmStringFree (label);
	}
	/*End of Only for Link and Image  */

	ok = XmFileSelectionBoxGetChild (W_AliDialog, XmDIALOG_OK_BUTTON);
	apply = XmSelectionBoxGetChild (W_AliDialog, XmDIALOG_APPLY_BUTTON);
	cancel = XmSelectionBoxGetChild (W_AliDialog, XmDIALOG_CANCEL_BUTTON);
	help = XmSelectionBoxGetChild (W_AliDialog, XmDIALOG_HELP_BUTTON);	 
	dialogtext = XmFileSelectionBoxGetChild (W_AliDialog, XmDIALOG_TEXT);

	XtUnmanageChild(help);
	XtUnmanageChild(apply);

     	XtAddCallback(cancel, XmNactivateCallback, AnchorLinkCancel_CB, flagadr);
	
	if (AnchorLinkImage != 1)
	{
		XtAddCallback(add, XmNactivateCallback, LinkImageAdd_CB, 
			(XtPointer) dialogtext);
		XtAddCallback(delete, XmNactivateCallback, LinkImageDelete_CB,
			(XtPointer) dialogtext);

		/*Only for Link  */
		if (AnchorLinkImage == 2)  
		{
			XtAddCallback(ok, XmNactivateCallback, LinkOk_CB, 
				(XtPointer) dialogtext);
		}
		else
		{
  		    	XtAddCallback (changebutton, XmNactivateCallback, 
				ChangeAlign_CB, NULL);	
			XtAddCallback(ok, XmNactivateCallback, ImageOk_CB, 
				(XtPointer) dialogtext);
		}
	}
	/*Only for Anchor  */
	else 
	{
		XtAddCallback(ok, XmNactivateCallback, AnchorOk_CB, 
			(XtPointer) dialogtext);
	}

	XtManageChild (W_AliDialog);
   	XtManageChild (frame);
}

/*----------------------------------------------------------------------*/
/*Anchor  */
/*----------------------------------------------------------------------*/

void 
Menu_LinkToName(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	AnchorLinkImage = 1;
	
	AliDialog(widget, &AnchorDialog_Flag);

	AnchorDialog_Flag=TRUE;
}  

/*----------------------------------------------------------------------*/
/*Link  */
/*----------------------------------------------------------------------*/

void 
Menu_LinkToFile(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	AnchorLinkImage = 2;

	AliDialog(widget, &LinkDialog_Flag);

	LinkDialog_Flag=TRUE;
} 
  
/*----------------------------------------------------------------------*/
/*Image  */
/*----------------------------------------------------------------------*/

void 
Menu_LinkToImage(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
	AnchorLinkImage = 3;

	AliDialog(widget, &ImageDialog_Flag);

	ImageDialog_Flag=TRUE;	
} 
    
/*----------------------------------------------------------------------*/
/*SetAnchorPoint  */
/*----------------------------------------------------------------------*/

void 
Menu_SetName(widget, client_data, call_data)
Widget	widget;		
XtPointer client_data, call_data;
{
 	char	*buffer1;               
 	char    *buffer2;

	buffer1 = "<A NAME=\"";
	buffer2 = "\">";
	
	InsertTags(buffer1, buffer2);
}
